<?php

require_once __DIR__ . "/../../vendor/autoload.php";

date_default_timezone_set("UTC");

/*
   |--------------------------------------------------------------------------
   | Create The Application
   |--------------------------------------------------------------------------
   |
   | Here we will load the environment and create the application instance
   | that serves as the central piece of this framework. We'll use this
   | application as an "IoC" container and router for this framework.
   |
 */

$app = new Laravel\Lumen\Application(dirname(__DIR__));

$app->withFacades();

$app->withEloquent();

/*
   |--------------------------------------------------------------------------
   | Register Container Bindings
   |--------------------------------------------------------------------------
   |
   | Now we will register a few bindings in the service container. We will
   | register the exception handler and the console kernel. You may add
   | your own bindings here if you like or you can make another file.
   |
 */

$app->singleton(Illuminate\Contracts\Debug\ExceptionHandler::class, progression\exceptions\Handler::class);

$app->singleton(Illuminate\Contracts\Console\Kernel::class, progression\console\Kernel::class);

/*
   |--------------------------------------------------------------------------
   | Register Config Files
   |--------------------------------------------------------------------------
   |
   | Now we will register the "app" configuration file. If the file exists in
   | your configuration directory it will be loaded; otherwise, we'll load
   | the default version. You may register other files below as needed.
   |
 */

$app->configure("app");
$app->configure("authentification");
$app->configure("version");
$app->configure("database");
$app->configure("jwt");
$app->configure("logging");
$app->configure("limites");
$app->configure("mail");
$app->configure("ldap");
$app->configure("exécuteur");

/*
   |--------------------------------------------------------------------------
   | Register Middleware
   |--------------------------------------------------------------------------
   |
   | Next, we will register the middleware with the application. These can
   | be global middleware that run before and after each request into a
   | route or middleware that'll be assigned to some specific routes.
   |
 */

//$app->middleware([
//    progression\http\middleware\Authenticate::class
//]);

$app->routeMiddleware([
	"auth" => progression\http\middleware\Authenticate::class,
]);

$app->routeMiddleware([
	"auth_optionnelle" => progression\http\middleware\AuthenticateOpt::class,
]);

$app->routeMiddleware([
	"permissionsRessources" => progression\http\middleware\PermissionsRessources::class,
]);

$app->routeMiddleware([
	"étatNonInactif" => progression\http\middleware\ÉtatNonInactif::class,
]);

$app->routeMiddleware([
	"étatValidé" => progression\http\middleware\ÉtatValidé::class,
]);

$app->routeMiddleware([
	"cors" => progression\http\middleware\Cors::class,
]);

$app->middleware([progression\http\middleware\Cors::class]);

/*
   |--------------------------------------------------------------------------
   | Register Service Providers
   |--------------------------------------------------------------------------
   |
   | Here we will register all of the application's service providers which
   | are used to bind services into the container. Service providers are
   | totally optional, so you are not required to uncomment this line.
   |
 */

// $app->register(App\Providers\AppServiceProvider::class);
$app->register(progression\providers\AuthServiceProvider::class);
$app->register(Illuminate\Redis\RedisServiceProvider::class);
$app->register(Illuminate\Mail\MailServiceProvider::class);

// $app->register(Tymon\JWTAuth\Providers\LumenServiceProvider::class);
// $app->register(App\Providers\EventServiceProvider::class);

/*
   |--------------------------------------------------------------------------
   | Load The Application Routes
   |--------------------------------------------------------------------------
   |
   | Next we will include the routes file so that they can all be added to
   | the application. This will provide all of the URLs the application
   | can respond to, as well as the controllers that may handle them.
   |
 */

$app->router->group(
	[
		"namespace" => "progression\http\contrôleur",
	],
	function ($router) {
		require __DIR__ . "/../routes/web.php";
	},
);

return $app;
