<?php
/*
   This file is part of Progression.

   Progression is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Progression is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Progression.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace progression\dao\question;

use DomainException;
use BadMethodCallException;
use progression\dao\EntitéDAO;
use progression\domaine\entité\question\{Question, QuestionProg, QuestionSys, QuestionSeq, Type};

class QuestionDAO extends EntitéDAO
{
	/**
	 * @param array<string> $includes;
	 */
	public function get_question($uri, array $includes = []): Question|null
	{
		$scheme = parse_url($uri, PHP_URL_SCHEME);

		if ($scheme == "file") {
			$infos_question = ChargeurFactory::get_instance()
				->get_chargeur_question_fichier()
				->récupérer_question($uri);
		} elseif ($scheme == "https") {
			$infos_question = ChargeurFactory::get_instance()->get_chargeur_question_http()->récupérer_question($uri);
		} else {
			throw new BadMethodCallException("Schéma d'URI invalide");
		}

		if ($infos_question === null) {
			return null;
		}

		$type = $infos_question["type"] ?? ($type = Type::PROG->value);
		if ($type == Type::PROG->value) {
			return DécodeurQuestionProg::load($infos_question);
		} elseif ($type == Type::SYS->value) {
			return DécodeurQuestionSys::load($infos_question);
		} elseif ($type == Type::SEQ->value) {
			return DécodeurQuestionSeq::load($infos_question, $includes);
		} else {
			throw new DomainException("Le fichier ne peut pas être décodé. Type inconnu");
		}
	}
}
